; *******************************
; Netflix Silverlight Control v2 Test6
; *******************************
;
; AutoHotKey script for control of Netflix: Movie Viewer in Internet Explorer


IniRead, AutoHideTimeout, settings.ini, settings, AutoHideTimeout, 2000
IniRead, FocusWindowTitle, settings.ini, settings, FocusWindowTitle, None
IniRead, UseFocusWindow, settings.ini, settings, UseFocusWindow, false
IniRead, HideFocusWindow, settings.ini, settings, HideFocusWindow, false
IniRead, HideTaskBar, settings.ini, settings, HideTaskBar, true
IniRead, UseDebug, settings.ini, settings, UseDebug, false

GroupAdd, FocusWindow, %FocusWindowTitle%

Gui, +Owner +ToolWindow -Caption
Gui, Color,Black,000000

DebugLog("Screen Width: " . A_ScreenWidth . ", Screen Height: " . A_ScreenHeight)

SysGet, MonCount, MonitorCount

Screen_Top := 0
Screen_Left := 0
Screen_Right := 0
Screen_Bottom := 0

Loop %MonCount%
{
    SysGet, Mon%A_Index%, Monitor, %A_Index%
    If (Mon%A_Index%Left <  Screen_Left)
        Screen_Left := Mon%A_Index%Left
    If (Mon%A_Index%Right >  Screen_Right)
        Screen_Right := Mon%A_Index%Right
    If (Mon%A_Index%Top <  Screen_Top)
        Screen_Top := Mon%A_Index%Top
    If (Mon%A_Index%Bottom >  Screen_Bottom)
        Screen_Bottom := Mon%A_Index%Bottom

    Mon%A_Index%Width := Abs(Mon%A_Index%Left - Mon%A_Index%Right)
    Mon%A_Index%Height := Abs(Mon%A_Index%Top - Mon%A_Index%Bottom)

    DebugLog("Monitor" . A_Index . " -- Left: " . Mon%A_Index%Left . " -- Top: " . Mon%A_Index%Top . " -- Right: " . Mon%A_Index%Right . " -- Bottom " . Mon%A_Index%Bottom . " -- Width " . Mon%A_Index%Width . " -- Height " . Mon%A_Index%Height)
}


Screen_Width := (Abs(Screen_Left) + Abs(Screen_Right))
Screen_Height := (Abs(Screen_Top) + Abs(Screen_Bottom))

Screen_XOffset := (Screen_Width - Screen_Right)
Screen_YOffset := (Screen_Height - Screen_Bottom)

; Start by searching all displays on a multi-display system
Search_Top := Screen_Top 
Search_Left := Screen_Left 
Search_Right := Screen_Right 
Search_Bottom := Screen_Bottom 

; Initialize window group
ViewerWindowTitle1 := "Netflix: Netflix Movie Viewer"
ViewerWindowTitle2 := "http://www.netflix.com/WiPlayer?movieid="
ViewerWindowTitle3 := "http://www.netflix.com/?movieid="
ViewerWindowTitle4 := "http://www.netflix.com - Netflix: Netflix Movie Viewer"

SetTitleMatchMode 2
DetectHiddenWindows On

GroupAdd, ViewerWindow, %ViewerWindowTitle1%
GroupAdd, ViewerWindow, %ViewerWindowTitle2%
GroupAdd, ViewerWindow, %ViewerWindowTitle3%

if WinExist("NetflixSilverlightControl")
{
    WinClose, NetflixSilverlightControl
    WinWaitClose, NetflixSilverlightControl
    DebugLog("Reset NetflixSilverlight Control")
}


if WinExist(A_ScriptFullPath)
{
    ; Window exists... rename it
    WinSetTitle, NetflixSilverlightControl
    DebugLog("Name NetflixSilverlightControl Window")
}
else
{
    ; The actual control script isn't running.  Run it.
    ;Run, %nscExe%
    DebugLog("Error: Unable to name NetflixSilverlightControl Window")
}


#Persistent
#NoEnv
#SingleInstance force

; Init onmessage

OnMessage(0x5555, "HandleMessage")

HandleMessage(wParam, lParam, msg)
{


    DebugLog("Rec Message - wParam:" . wParam . " lParam:" . lParam)

    if (wParam = 10)
    {

        ; Decide the action to take
        ; (AHK has no switch/case so, we'll have to use else if)
        if (lParam = 100)
        {
            Gosub, CloseViewer ; Exit the netflix movie viewer, restart this script, and wait for a new viewer window to open
        }
        else if (lParam = 101)
        {
            Gosub, JumpAhead
        }
        else if (lParam = 102)
        {
            Gosub, JumpBack ; Jump back in the timeline
        }
        else if (lParam = 103)
        {
            Gosub, NextEpisode ; Move to next Chapter/Episode
        }
        else if (lParam = 104)
        {
            Gosub, PrevEpisode ; Move to previous Chapter/Episode
        }
        else if (lParam = 105)
        {
            Gosub, Play ; Play
        }
        else if (lParam = 106)
        {
            Gosub, Pause ; Pause
        }
        else if (lParam = 107)
        {
            Gosub, TogglePause ;Toggle play/pause
        }
        else if (lParam = 108)
        {
            Gosub, ToggleMute ; Toggle Mute On/Off
        }
        else if (lParam = 109)
        {
            Gosub, VolumeUp ; Move volume Up 
        }
        else if (lParam = 110)
        {
            Gosub, VolumeDown ; Move volume down
        }


    }

    ; Since returning quickly is often important, it is better to use a ToolTip than
    ; something like MsgBox that would prevent the function from finishing:
    ;ToolTip Message %msg% arrived:`nWPARAM: %wParam%`nLPARAM: %lParam%
    ;MsgBox,  Message %msg% arrived:`nWPARAM: %wParam%`nLPARAM: %lParam%
}


; Start of script main loop.
; ***************************
ScriptStart:



; initialize variables
ScriptMode := 0
TimePosition := 0
TimeMovementBuffer := 0
TimeMovementCount := 0
TimeMovementMode := 0


NavBarHeight := 72

Gosub, NotifyClose

DebugLog("Waiting for Netflix Movie Viewer")

; Wait for a Netflix Movie Viewer window to open
SetTitleMatchMode 2
DetectHiddenWindows On
WinWait, ahk_group ViewerWindow

BlockInput, On

HideHost()

Gosub, NotifyOpen

DebugLog("Found Netflix Movie Viewer Window")

; Set timer to check if window closes
SetTimer, CheckViewerWindow, 100


DebugLog("Searching for nf_fs.bmp")
; Buffer events
#MaxThreadsBuffer On
;
; Make sure the player window is active
WinActivate, ahk_group ViewerWindow

; Wait up to 60 seconds for the "Full screen" button
EnterFullScreen := GetButtonPos("nf_fs.bmp", 59, 16, 600, 0)
; For multi-display system, find the monitor that the EFS button was found on.
ActiveMonitor := GetMonitor(EnterFullScreen)
DebugLog("Active Monitor: " . ActiveMonitor)
Active_Width := Mon%ActiveMonitor%Width
Active_Height := Mon%ActiveMonitor%Height
; Restrict further button searching to the monitor the player was found on
Search_Top := Mon%ActiveMonitor%Top 
Search_Left := Mon%ActiveMonitor%Left 
Search_Right := Mon%ActiveMonitor%Right 
Search_Bottom := Mon%ActiveMonitor%Bottom 


        TopLoc := Mon%ActiveMonitor%Top
        LeftLoc := Mon%ActiveMonitor%Left


        Gui,Show, x%LeftLoc% y%TopLoc% h%Active_Height% w%Active_Width%

; Move mouse to the FS button, and get info about the control it's under
HideTaskBar()
WinActivate, ahk_group ViewerWindow
MoveToPos(EnterFullScreen)
MouseGetPos,,, PlayerWindowId, PlayerControlName
HideMouse()
WinSet, AlwaysOnTop, On, ahk_id %playerWindowId%
WinSet, Top,, ahk_id %playerWindowId%
WinSet, AlwaysOnTop, On, ahk_id %playerWindowId%
DebugLog(PlayerControlName)

WinGetPos, WinPosX,WinPosY,WinWidth,WinHeight, ahk_id %playerWindowId%

; Store our original settings so they can be undone on exit/close 
OriginalPosX := WinPosX
OriginalPosY := WinPosY
OriginalWidth := WinWidth
OriginalHeight := WinHeight

OnExit, DoExit

Gosub, SetNavigation

if ((WinDiffY > 1) && (WinDiffX > WinDiffY)){

    result := ResizeWindow((NavPosX - Round(WinDiffX / 2)), (NavPosY - Round(WinDiffY / 2)), NavWidth, (NavHeight + WinDiffY))
    Gosub, SetNavigation
}

DebugLog("Nav Size: " . NavPosX . "," . NavPosY . "," . NavWidth . "," . NavHeight . " Aspect: " . MovieAspect . " / " . PlayerAspect . " / " . WinAspect )

;result := ResizeWindow(NavPosX, NavPosY, NavWidth, NavHeight)

;WinSet, Region, %PosX%-%PosY% W%PlayerWidth% H%PlayerHeight%, ahk_id %playerWindowId%

ShowNavigation()

;DebugLog("Stuff: " . (WinPosX - PosX) . ",  "PosX . "," . PosY . "," . PlayerWidth . "," . PlayerHeight)

Sleep, 500

; Wait up to 40 seconds for the "Full screen" button, in its new position
EnterFullScreen := GetButtonPos("nf_fs.bmp", 59, 16, 400, 0)

NfsBarPos := (GetY(EnterFullScreen) - 20)

Search_Top := NfsBarPos


DebugLog("Searching for nf_pause_a.bmp or nf_pause_b.bmp")
; now find the pause button
Loop, 600
{
    result =: WiggleMouse()
    Pause := GetButtonPos("nf_pause_a.bmp", 57, 21, 1, 45)
        if (Pause > 0)
           break
    Pause := GetButtonPos("nf_pause_b.bmp", 57, 21, 1, 45)
        if (Pause > 0)
            break
    Pause := GetButtonPos("nf_play_a.bmp", 57, 21, 1, 45)
        if (Pause > 0)
            break
    Pause := GetButtonPos("nf_play_b.bmp", 57, 21, 1, 45)
        if (Pause > 0)
            break
    Sleep, 100
}

DebugLog("Searching for nf_vol_(a/b/c).bmp")
; and now look for the volume button in all its various forms
Loop, 600
{
    result =: WiggleMouse()
    Vol := GetButtonPos("nf_vol_a.bmp", 20, 20, 1, 45)
        if (Vol > 0)
           break
    Vol := GetButtonPos("nf_vol_b.bmp", 20, 20, 1, 45)
        if (Vol > 0)
            break
    Vol := GetButtonPos("nf_vol_c.bmp", 20, 20, 1, 45)
        if (Vol > 0)
            break
    Sleep, 100
}

; If this a a TV series, there could be chapter buttons.  Find them
PreviousChapter := GetButtonPos("nf_prev_chap.bmp", 15, 11, 1, NfsBarPos)
NextChapter := GetButtonPos("nf_next_chap.bmp", 15, 11, 1, NfsBarPos)

; Estimate the width of the timeline bar for later use in jumping forward/back
; We base it off the position of the pause and enter full screen buttons
FarLeftTime := (GetX(Pause) + 56)
TimeLine := (GetY(Pause) - 6)
FarRightTime := (GetX(EnterFullScreen) - 98)

; Let the hot key events know that we have everything set up, and they can now run
ScriptMode := 1

; Move the mouse to the lower right corner
result := HideMouse()

; Size the player window with navigation

DebugLog("Player Size: " . PosX . "," . PosY . "," . PlayerWidth . "," . PlayerHeight . "," . Active_Width . "," . Active_Height)


result := ResizeWindow(0, 0, MovWidth, MovHeight)

ControlGetPos,PosX,PosY,PlayerWidth,PlayerHeight, %PlayerControlName%, ahk_id %playerWindowId%

MovieHeight := (PlayerHeight - NavBarHeight)

MovPosX := Mon%ActiveMonitor%Left
MovPosY := Mon%ActiveMonitor%Top
MovPosX -= ( PosX )
MovPosY -= ( PosY )


if ( MovieHeight < Active_Height )
{
    MovPosY += Round((Active_Height / 2) - Round(MovieHeight / 2))
}

if ( PlayerWidth < Active_Width )
{
    MovPosX += Round((Active_Width / 2) - Round(PlayerWidth / 2))
}

DebugLog("Mov Size: " . MovPosX . "," . MovPosY . "," . MovWidth . "," . MovHeight . " Aspect: " . MovieAspect . " / " . PlayerAspect . " / " . WinAspect )


GoFullScreen()

DebugLog("Waiting For Messages...")

return

; **********************************
; Netflix Viewer Control Subroutines
; **********************************
;
; Call with "Gosub, <name>"
; Example:
; Gosub, VolumeUp
;


; Exit the netflix movie viewer, and wait
; for a new viewer window to open
; ******************************************
CloseViewer:

    SetTimer, HideAfterTime, Off
    BlockInput, On
    CleanUp()
    BlockInput, On
    SetTitleMatchMode 2
    DetectHiddenWindows On
    IfWinExist, ahk_group ViewerWindow
    {
        WinClose, ahk_group ViewerWindow
        WinWaitClose, ahk_group ViewerWindow
    }
    FocusHost()
    Gosub, NotifyClose
    Reload

return

; Jump ahead about 10 pixels in the timeline
; ******************************************
JumpAhead:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1 && ScriptMode > -1)
        return
    if (TimeMovementCount > 5 || ScriptMode = 1)
    {
        TimeMovementBuffer += 2
    }
    else
    {
        TimeMovementBuffer += 10
    }
    SetTimer, DoTimeScroll, 50

return

; Jump back about 10 pixels in the timeline
; ******************************************
JumpBack:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1 && ScriptMode > -1)
        return
    if (TimeMovementCount > 5 || ScriptMode = 1)
    {
        TimeMovementBuffer -= 2
    }
    else
    {
        TimeMovementBuffer -= 10
    }
    SetTimer, DoTimeScroll, 50

return

; Move to next Chapter/Episode
; *****************************
NextEpisode:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1)
        return

        CoordMode Mouse
        result := WaitForBar()
        ;result := ClickPos(ExitFullScreen)
        ; Wait for the chapter button to become visible
        NextChapter := GetButtonPos("nf_next_chap.bmp", 15, 11, 2, 0)
        result := ClickPos(NextChapter)
        if (NextChapter > 0)
        {
            ; Wait 2 seconds for the new video to load
            sleep 2000
            ; enter back into full screen
            result := GetButtonPos("nf_fs.bmp", 59, 16, 200, 0)
            ;PreviousChapter := GetButtonPos("nf_prev_chap.bmp", 15, 11, 2, 0)
	    ;NextChapter := GetButtonPos("nf_next_chap.bmp", 15, 11, 2, 0)
            ;result := ClickPos(EnterFullScreen)
        }

        SetTimer, HideAfterTime, %AutoHideTimeout%


return

; Move to previous Chapter/Episode
; *********************************
PrevEpisode:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1)
        return

    CoordMode Mouse
    result := WaitForBar()
    ;result := ClickPos(ExitFullScreen)
    ; Wait for the chapter button to become visible
    PreviousChapter := GetButtonPos("nf_prev_chap.bmp", 15, 11, 2, 0)
    if (PreviousChapter > 0)
    {
        result := ClickPos(PreviousChapter)
        ; Wait 2 seconds for the new video to load
        sleep 2000
        ; enter back into full screen
        result := GetButtonPos("nf_fs.bmp", 59, 16, 200, 0)
        ;PreviousChapter := GetButtonPos("nf_prev_chap.bmp", 15, 11, 2, 0)
	;NextChapter := GetButtonPos("nf_next_chap.bmp", 15, 11, 2, 0)
        ;result := ClickPos(EnterFullScreen)
    }
    SetTimer, HideAfterTime, %AutoHideTimeout%

return

; Play
; *****************************
Play:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1)
        return
    CoordMode Mouse
    result := WaitForBar()
    Play := GetButtonPos("nf_play_a.bmp", 57, 21, 1, 0)
    if (Play > 0)
    {
        result := ClickPos(Play)
    }

    SetTimer, HideAfterTime, %AutoHideTimeout%

return

; Pause
; *****************************
Pause:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1)
        return
    CoordMode Mouse
    result := WaitForBar()
    Button := GetButtonPos("nf_pause_a.bmp", 57, 21, 1, 45)
        if (Button > 0)
        {
            result := ClickPos(Button)
            return
        }
    Button := GetButtonPos("nf_pause_b.bmp", 57, 21, 1, 45)
        if (Button > 0)
        {
            result := ClickPos(Button)
            return
        }

    SetTimer, HideAfterTime, %AutoHideTimeout%

return

; Toggle play/pause
; *****************************
TogglePause:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1)
        return
    ;CoordMode Mouse
    result := WaitForBar()
    result := ClickPos(Pause)
    ;ClickControlPos(Pause)

    SetTimer, HideAfterTime, %AutoHideTimeout%

return

; Toggle Mute On/Off
; *****************************
ToggleMute:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1)
        return
    CoordMode Mouse
    result := WaitForBar()
    result := ClickPos(Vol)

    SetTimer, HideAfterTime, %AutoHideTimeout%

return

; Move volume Up
; *****************************
VolumeUp:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1)
        return

    VolPos := GetVolumeBarPos(Vol)

    CoordMode Mouse
    if (VolPos > 0)
    {
        x := GetX(Vol)
        NewVolPos := (VolPos - 5)
        BlockInput, On
        MouseMove, %x%, %VolPos%, 5
        MouseClickDrag, Left, x, VolPos, x, NewVolPos, 5
        FocusHost()
    }

    SetTimer, HideAfterTime, %AutoHideTimeout%

return

; Move volume down
; *****************************
VolumeDown:

    SetTimer, HideAfterTime, Off

    if (ScriptMode < 1)
        return

    VolPos := GetVolumeBarPos(Vol)

    CoordMode Mouse
    if (VolPos > 0)
    {
        x := GetX(Vol)
        NewVolPos := (VolPos + 5)
        BlockInput, On
        MouseMove, %x%, %VolPos%, 3
        MouseClickDrag, Left, x, VolPos, x, NewVolPos, 3
        FocusHost()
    }

    SetTimer, HideAfterTime, %AutoHideTimeout%

return


; *******************
; Timer Subroutines
; *******************

; Used by check viewer window timer
; Perodically check if the viewer window is open
; If not, re-start from the top
CheckViewerWindow:

    SetTitleMatchMode 2
    DetectHiddenWindows On

    IfWinNotExist, ahk_group ViewerWindow
    {
        CleanUp()
        DebugLog("Netflix Movie Viewer Window no longer exists.  Resetting NetflixSilverlightControl")
        Reload
    }

return


HideAfterTime:


    BlockInput, Default
    SetTimer,HideAfterTime,Off
    GoFullScreen()
    HideMouse()

return

; Used by jump ahead/back hotkeys
DoTimeScroll:

    if ((TimeMovementBuffer = 0) | ((((TimePosition + TimeMovementBuffer) > FarRightTime)  | ((TimePosition + TimeMovementBuffer) < FarLeftTime)) & (TimeMovementMode = 1)))
        {
            TimePosition += TimeMovementBuffer
            TimeMovementBuffer := 0
            if (TimePosition > FarRightTime)
            {
                TimePosition := (FarRightTime)
            }
            if (TimePosition < FarLeftTime)
            {
                TimePosition := (FarLeftTime)
            }
            ; Hiding the mouse, and then repositioning it
            ; keeps the navigation bar active while waiting for
            ; the movment timer to time out
            ;result := HideMouse()
            CoordMode Mouse
            ;BlockInput, On
            MouseMove, %TimePosition%, %TimeLine%, 3
            
            TimeMovementCount += 1
            if (TimeMovementCount > 20)
            {
                CoordMode Mouse
                Click up
                ;MouseMove, %TimePosition%, %TimeLine%, 0
                ;result := WaitForBar()
                ;CoordMode Mouse
                ;Click, %TimePosition%, %TimeLine%
                ScriptMode := 1
                TimeMovementMode := 0
                SetTimer,DoTimeScroll,Off
                HideMouse()
                GoFullScreen()
                FocusHost()
            }
        }
        else
        {
            ScriptMode := -1
            TimeMovementCount  := 0
            if (TimeMovementMode = 0)
            {
                TimeMovementMode := 1
                result := WaitForBar()

                DebugLog("Looking for nf_mark_[a|b].bmp")
                Loop, 10
                {
                    result =: WiggleMouse()
                    CurPos := GetButtonPos("nf_mark_a.bmp", 2, 10, 1, 45)
                        if (CurPos > 0)
                           break
                    CurPos := GetButtonPos("nf_mark_b.bmp", 2, 10, 1, 45)
                        if (CurPos > 0)
                           break
                    Sleep, 100
                }
                ;CurPos := GetButtonPos("nf_mark.bmp", 2, 10, 500, 45)
                TimePosition := GetX(CurPos)
            }
            CoordMode Mouse
            MouseMove, %TimePosition%, %TimeLine% , 0
            BlockInput, On
            Click down
            FocusHost()
            TimePosition += TimeMovementBuffer
            TimeMovementBuffer := 0
            if (TimePosition > FarRightTime)
            {
                TimePosition := (FarRightTime)
            }
            if (TimePosition < FarLeftTime)
            {
                TimePosition := (FarLeftTime)
            }
            ;BlockInput, On
            MouseMove, %TimePosition%, %TimeLine% , 3
            ;FocusHost()
    }
   
return

DoExit:

    CleanUp()

ExitApp

; *************************
; Used to notify EventGhost
; *************************

NotifyClose:

    SetTitleMatchMode 1
    DetectHiddenWindows On
  
    ifWinExist, EventGhost ahk_class HiddenMessageReceiver
    {
        WinGet, pID, PID
        EventGhostPath := GetModuleFileNameEx(pID)
        Run, %EventGhostPath% -e NetflixSilverlightControl.PlayerClose
    }

return


NotifyOpen:

    SetTitleMatchMode 1
    DetectHiddenWindows On
  
    ifWinExist, EventGhost ahk_class HiddenMessageReceiver
    {
        WinGet, pID, PID
        EventGhostPath := GetModuleFileNameEx(pID)
        Run, %EventGhostPath% -e NetflixSilverlightControl.PlayerOpen
    }

return


; ****************************
; Functions
; ****************************


; Look for position on an image
GetButtonPos(g_image,g_width,g_height,g_wait,y_start)
{

    global Screen_XOffset
    global Screen_YOffset
    global Search_Top
    global Search_Left
    global Search_Bottom
    global Search_Right

    full_image_path := A_ScriptDir . "\netflix_player_buttons\" . g_image
    reslt = 0
    CoordMode Pixel
    Loop, %g_wait%
    {
        ImageSearch, FoundX, FoundY, %Search_Left%, %Search_Top%, %Search_Right%, %Search_Bottom%, %full_image_path%
        if ErrorLevel = 0
        {
             ; Pack X,Y coords as two 16 bit integers in one 32 bit word.
             ; We do this because autohotkey does not yet support real arrays,
             ; and we can only return one value from this function
             x := ((FoundX + (g_width/2)) + Screen_XOffset)
             y := (((Foundy + (g_height/2)) + Screen_YOffset) << 16)
             reslt := (y | x)
             break
        }
        else
        {
            if (g_wait > 1)
                Sleep, 100
        }
    }
    if (reslt > 0)
    {
        DebugLog("Found " . g_image . " at location " . reslt)
    }
    return reslt
}


; Move to position
MoveToPos(position)
{

    global Screen_XOffset
    global Screen_YOffset

    x := ((position & 65535) - Screen_XOffset)
    y := ((position >> 16) - Screen_YOffset)
    CoordMode Mouse
    MouseMove, %x%, %y%, 0
    return 0
}


; Move mouse to lower right corner of screen
HideMouse()
{

    global Screen_Bottom
    global Screen_Right

    CoordMode Mouse
    MouseMove, %Screen_Right%, %Screen_Bottom%, 0
    return 0
}


; Click a position, them move the mouse to the lower right
; corner of the screen 
ClickPos(position){
    global Screen_XOffset
    global Screen_YOffset

    x := ((position & 65535) - Screen_XOffset)
    y := ((position >> 16) - Screen_YOffset)
    CoordMode Mouse
    MouseMove, %x%, %y%, 0
    ;MouseMove, %x%, %y%, 1
    CoordMode Mouse
    BlockInput, On
    Click, %x%, %y%
    FocusHost()
    HideMouse()
    DebugLog("Clicked position " . position)
    return 0
}


; Get the "X" portion of a position
GetX(position)
{
    global Screen_XOffset
    global Screen_YOffset

    x := ((position & 65535) - Screen_XOffset)
    y := ((position >> 16) - Screen_YOffset)
    return x
}


; Get the "Y" portion of a position
GetY(position)
{
    global Screen_XOffset
    global Screen_YOffset

    x := ((position & 65535) - Screen_XOffset)
    y := ((position >> 16) - Screen_YOffset)
    return y
}


; Move the mouse cursor around a bit in order to bring up
; the netflix navigation bar that auto hides
WiggleMouse()
{
    ;wiggle := (A_ScreenWidth - 10)
    ;CoordMode Mouse
    ;MouseMove, %A_ScreenWidth%, %A_ScreenHeight%, 2
    ;CoordMode Mouse
    ;MouseMove, %wiggle%, %A_ScreenHeight%, 2 
    ;CoordMode Mouse
    ;MouseMove, %A_ScreenWidth%, %A_ScreenHeight%, 2
    ;CoordMode Mouse
    ;MouseMove, %wiggle%, %A_ScreenHeight%, 1 
    return 0
}


; See of the netflix navigation bar is visible
; and bring it up, if not
WaitForBar()
{

    BlockInput, Mouse

    SetTimer,HideAfterTime,Off
    ShowNavigation()

    result := LookForVolumeButton()
    if (result = 0)
    {
  
        result := WiggleMouse()

        Loop, 50
        {
            result := LookForVolumeButton()
            if (result = 1)
                break
            Sleep, 50       
        }
        ;Sleep, 100
        return result
    }
    return result
}


; See of the volume button is visible
; (if it is, we know the navigation bar is visible)
LookForVolumeButton()
{

    global Screen_XOffset
    global Screen_YOffset
    global Search_Top
    global Search_Left
    global Search_Bottom
    global Search_Right

   full_image_path_a := A_ScriptDir . "\netflix_player_buttons\nf_vol_a.bmp"
   full_image_path_b := A_ScriptDir . "\netflix_player_buttons\nf_vol_b.bmp"
   full_image_path_c := A_ScriptDir . "\netflix_player_buttons\nf_vol_c.bmp"

   ;y_start := (A_ScreenHeight - 50)
   ;y_start := 0

       CoordMode Pixel
       ImageSearch, FoundX, FoundY, %Search_Left%, %Search_Top%, %Search_Right%, %Search_Bottom%, %full_image_path_a%
           if ErrorLevel = 0
               return 1
       ImageSearch, FoundX, FoundY, %Search_Left%, %Search_Top%, %Search_Right%, %Search_Bottom%, %full_image_path_b%
           if ErrorLevel = 0
               return 1
       ImageSearch, FoundX, FoundY, %Search_Left%, %Search_Top%, %Search_Right%, %Search_Bottom%, %full_image_path_c%
           if ErrorLevel = 0
               return 1
    return 0

}


; Check if the volume slider bar is visible and return its position
; If its not visible, then make it visible
GetVolumeBarPos(Vol){
    result := LookForVolumeButton()
    if (result = 0)
    {
        result := WaitForBar()
    }

    VolPos := FindVolumeBar(Vol)
    if (VolPos = 0)
    {
        ; Look for volume bar
        result := MoveToPos(Vol)
        Loop, 20
        {
            VolPos := FindVolumeBar(Vol)
            if (VolPos > 0)
                break
            Sleep, 100
        }
    }
    return VolPos
}


; Return the position of the volume slider bar.  
; If its not visible return 0
FindVolumeBar(Vol){

    x := GetX(Vol)
    y := GetY(Vol)
    x1 := (x - 12)
    x2 := (x + 12)
    y1 := (y - 74)
    y2 := (y - 16)
    full_image_path_a := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_a.bmp"
    full_image_path_b := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_b.bmp"
    full_image_path_c := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_c.bmp"
    full_image_path_d := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_d.bmp"
    full_image_path_e := A_ScriptDir . "\netflix_player_buttons\nf_vol_slide_e.bmp"

    VolPos := 0

        CoordMode Pixel
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_a%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_b%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_c%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_d%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }
        ImageSearch, FoundX, FoundY, %x1%, %y1%, %x2%, %y2%, %full_image_path_e%
        if ErrorLevel = 0
        {
            VolPos := (FoundY + 1)
            return VolPos
        }

    return VolPos
}



GetModuleFileNameEx( p_pid )
{
   if A_OSVersion in WIN_95,WIN_98,WIN_ME
   {
      MsgBox, This Windows version (%A_OSVersion%) is not supported.
      return
   }

   h_process := DllCall( "OpenProcess", "uint", 0x10|0x400, "int", false, "uint", p_pid )
   if ( ErrorLevel or h_process = 0 )
      return
   
   name_size = 255
   VarSetCapacity( name, name_size )
   
   result := DllCall( "psapi.dll\GetModuleFileNameExA", "uint", h_process, "uint", 0, "str", name, "uint", name_size )
   
   DllCall( "CloseHandle", h_process )
   
   return, name
} 


ResizeWindow(X, Y, Width, Height)
{

    global playerWindowId
/*
    WinSet, AlwaysOnTop, On, ahk_id %playerWindowId%
    WinSet, Top,, ahk_id %playerWindowId%
    WinSet, AlwaysOnTop, On, ahk_id %playerWindowId%
*/

    BlockInput, On
    Loop, 20
    {

    hWnd := WinExist(ahk_id %playerWindowId%)

    DllCall("SetWindowPos", "UInt", playerWindowId, "UInt", 0, "Int", X, "Int", Y, "Int", Width, "Int", Height, "UInt", 0x0400) ;SWP_NOSENDCHANGING (override size limmit of window in vista)

    result := 0
    ; Wait untill the window resizes.
    ;Sleep, 500

        WinGetPos, WinX,WinY,WinW,WinH, ahk_id %playerWindowId%

        If ((WinW = Width) && (WinH = Height) && (WinX = X) && (WinY = Y))
        {
            result := 1
            break
        }

        Sleep, 50
    }
    FocusHost()

    return result   

}


GoFullScreen()
{

    global MovPosX
    global MovPosY
    global MovWidth
    global MovHeight
    global PlayerControlName
    global playerWindowId
    global NavBarHeight
    global Active_Width
    global Active_Height
    global ActiveMonitor
    SysGet, Mon%ActiveMonitor%, Monitor, %ActiveMonitor%

    ResizeWindow(MovPosX, MovPosY, MovWidth, MovHeight)
    ControlGetPos,PosX,PosY,PlayerWidth,PlayerHeight, %PlayerControlName%, ahk_id %playerWindowId%
    MovieHeight := (PlayerHeight - NavBarHeight)

    if (PlayerWidth > Active_Width)
    {
        PlayerWidth := Active_Width
    }
    if (MovieHeight > Active_Height)
    {
        MovieHeight := Active_Height
    }

    WinSet, Region, %PosX%-%PosY% W%PlayerWidth% H%MovieHeight%, ahk_id %playerWindowId%
   
   return
}


ShowNavigation()
{

    global NavPosX
    global NavPosY
    global NavWidth
    global NavHeight
    global PlayerControlName
    global playerWindowId
    global NavBarHeight
    global Active_Width
    global Active_Height
    global ActiveMonitor
    global PlayerXLoc
    global PlayerYLoc
    SysGet, Mon%ActiveMonitor%, Monitor, %ActiveMonitor%

    ResizeWindow(NavPosX, NavPosY, NavWidth, NavHeight)
    ControlGetPos,PosX,PosY,PlayerWidth,PlayerHeight, %PlayerControlName%, ahk_id %playerWindowId%
    MovieHeight := (PlayerHeight - NavBarHeight)

    if (PlayerWidth > Active_Width)
    {
        PlayerWidth := Active_Width
    }
    if (PlayerHeight > Active_Height)
    {
        PlayerHeight := Active_Height
    }

    WinSet, Region, %PosX%-%PosY% W%PlayerWidth% H%PlayerHeight%, ahk_id %playerWindowId%

    return
    
}


GetMonitor( position )
{

    SysGet, MonCount, MonitorCount

    gmx := GetX(position)
    gmy := GetY(position)

    DebugLog("Searching (" . MonCount . ") Monitors for " . gmx . "/" . gmy)

    gmr := 0

    Loop %MonCount%
    {
        SysGet, Mon%A_Index%, Monitor, %A_Index%

        DebugLog("Searching Mon" . A_Index . ": L" . Mon%A_Index%Left . " R" . Mon%A_Index%Right . " T" . Mon%A_Index%Top . "  B" . Mon%A_Index%Bottom)

        if ((gmx >= Mon%A_Index%Left) && (gmx < Mon%A_Index%Right) && (gmy >= Mon%A_Index%Top) && (gmy < Mon%A_Index%Bottom))
        {
            DebugLog("Found on Mon" . A_Index)
            gmr := A_Index
            break
        }
    }
    return gmr

}


HideTaskBar()
{

    global HideTaskBar

    If (HideTaskBar = "true")
    {
        WinHide ahk_class Shell_TrayWnd
        WinHide Start ahk_class Button
    }
    return
}

ShowTaskBar()
{

    global HideTaskBar

    If (HideTaskBar = "true")
    {
        WinShow ahk_class Shell_TrayWnd
        WinShow Start ahk_class Button
    }
    return
}


; Clean up our mess  (used on exit, etc.)
CleanUp()
{
    global playerWindowId
    global OriginalPosX
    global OriginalPosY
    global OriginalWidth
    global OriginalHeight

    BlockInput, Default
    BlockInput, On
    
    ScriptMode := 0
    SetTimer, CheckViewerWindow, Off
    SetTimer, DoTimeScroll, Off

    WinActivate, ahk_id %playerWindowId%
    WinSet, Region,, ahk_id %playerWindowId%
    WinMove, ahk_id %playerWindowId%,, %OriginalPosX%, %OriginalPosY%, %OriginalWidth%, %OriginalHeight%
    WinSet, AlwaysOnTop, Off, ahk_id %playerWindowId%
    Gui, Destroy
    Gui, 2:Destroy
    ShowTaskBar()
    ShowHost()
    FocusHost()

}

;Transition focus to the requested host window
FocusHost()
{
    global FocusWindow
    global UseFocusWindow
    if (UseFocusWindow = "true")
    {
        WinActivate, ahk_group FocusWindow
    }
    BlockInput, Off
}

HideHost()
{
    global FocusWindow
    global UseFocusWindow
    global HideFocusWindow
    global HostX
    global HostY
    global HostTrans
    global Screen_Right
    global Screen_Bottom

    if ((HideFocusWindow = "true") && (UseFocusWindow = "true"))
    {
        WinGet, Trans, Transparent, ahk_group FocusWindow
        WinGetPos, X, Y, Width, Height, ahk_group FocusWindow
        ;WinSet, Transparent, 0, ahk_group FocusWindow
        WinMove, ahk_group FocusWindow,, %Screen_Right%, %Screen_Bottom%
    }

    HostX := X
    HostY := Y
    HostTrans := Trans

}

ShowHost()
{
    global FocusWindow
    global UseFocusWindow
    global HideFocusWindow
    global HostX
    global HostY
    global HostTrans

    if ((HideFocusWindow = "true") && (UseFocusWindow = "true"))
    {
        ;WinSet, Transparent, %HostTrans%, ahk_group FocusWindow
        WinMove, ahk_group FocusWindow,, %HostX%, %HostY%
    }

}

DebugLog( LogMessage )
{
    global UseDebug
    LogFile := "debuglog.txt"
    if (useDebug = "true")
    {
    FileAppend,
(
%A_Hour%:%A_Min%:%A_Sec%:%A_MSec% %A_MM%/%A_DD%/%A_YYYY% - %LogMessage%

), %LogFile%
    }
}



SetNavigation:


ControlGetPos,PosX,PosY,PlayerWidth,PlayerHeight, %PlayerControlName%, ahk_id %playerWindowId% 
MovieHeight := (PlayerHeight  - NavBarHeight)

WinGetPos, WinPosX,WinPosY,WinWidth,WinHeight, ahk_id %playerWindowId%

DebugLog("Window Size: " . WinPosX . "," . WinPosY . "," . WinWidth . "," . WinHeight . "," . Active_Width . "," . Active_Height)
DebugLog("Player Size: " . PosX . "," . PosY . "," . PlayerWidth . "," . PlayerHeight . "," . Active_Width . "," . Active_Height)

MovieAspect := (PlayerWidth / MovieHeight)
PlayerAspect := (PlayerWidth / PlayerHeight)
WinAspect := (Active_Width / Active_Height)

;Calculate new size of player window with navigation
if (PlayerAspect < WinAspect)
{
    NavWidth := (Ceil(Active_Height * PlayerAspect))
    NavHeight := (Active_Height)
}
else
{
    NavWidth := (A_ScreenWidth)
    NavHeight := (Ceil(Active_Width / PlayerAspect))
}

TargetWidth := NavWidth
TargetHeight := NavHeight

DebugLog("New Player Size: " . NavWidth . "," . NavHeight)

NavWidth += (WinWidth - PlayerWidth)
NavHeight += (WinHeight - PlayerHeight)

;Calculate new size of player window without navigation
if (MovieAspect < WinAspect)
{
    MovWidth := (Ceil(Active_Height * MovieAspect))
    MovHeight := (Active_Height)
}
else
{
    MovWidth := (Active_Width)
    MovHeight := (Ceil(Active_Width / MovieAspect))
}


DebugLog("New Movie Size: " . MovWidth . "," . MovHeight)

MovWidth += (WinWidth - PlayerWidth)
MovHeight += (WinHeight - MovieHeight)

;NavPosX := Mon%ActiveMonitor%Left
;NavPosY := Mon%ActiveMonitor%Top

;NavPosX -= ( PosX - Round((WinWidth - PlayerWidth)/2))
;NavPosY -= ( PosY - Round((WinHeight - PlayerHeight)/2))

result := ResizeWindow(0, 0, NavWidth, NavHeight)

ControlGetPos,PosX,PosY,PlayerWidth,PlayerHeight, %PlayerControlName%, ahk_id %playerWindowId%
WinGetPos, WinPosX,WinPosY,WinWidth,WinHeight, ahk_id %playerWindowId%

WinDiffY := (Mon%ActiveMonitor%Height - PlayerHeight)
WinDiffX := (Mon%ActiveMonitor%Width - PlayerWidth)

DebugLog("Window Size: " . WinPosX . "," . WinPosY . "," . WinWidth . "," . WinHeight)

NavPosX := Mon%ActiveMonitor%Left
NavPosY := Mon%ActiveMonitor%Top
DebugLog("Nav Size: " . NavPosX . "," . NavPosY . "," . NavWidth . "," . NavHeight . " Aspect: " . MovieAspect . " / " . PlayerAspect . " / " . WinAspect )

NavPosX -= ( PosX )
NavPosY -= ( PosY )


if ( PlayerHeight < Active_Height )
{
   if ( (PlayerHeight + (NavBarHeight / 2)) < Active_Height )
   {
       NavPosY += Round((Active_Height / 2) - Round((PlayerHeight - NavBarHeight) / 2))
   }
   else
   {
       NavPosY += Round((Active_Height / 2) - Round(PlayerHeight / 2))
   }
}

if ( PlayerWidth < Active_Width )
{
    NavPosX += Round((Active_Width / 2) - Round(PlayerWidth / 2))
}



return
